#!/usr/bin/env python3
# -*- coding:utf-8 -*-
import numpy as np
from ctypes import c_int, c_double, POINTER, CDLL
import typing
from . import band as _bd
from .typeDefs import doubleArray, intArray, floatOrArray
import os
path = os.path.dirname(__file__)
__all__ = ['cSimpleSolve1D', 'cSimpleFillPsi',
           'Band', 'cBandFillPsi', 'cBandSolve1D',
           'cLOphononScatter', 'cLOtotal']


def bindOpenMP(on: bool = True) -> typing.Tuple[CDLL, typing.Type[object]]:
    """
    set OpenMP. Default = True
    """
    global _clib
    if(on):
        _clib = np.ctypeslib.load_library('1DSchrodinger_MP', path)
    else:
        _clib = np.ctypeslib.load_library('1DSchrodinger', path)

    _bd.init(_clib)
    global Band
    from .band import cBand, Band
    #  _clib.Numerov.argtypes = [c_double, c_int, c_double, c_double,
    #                            c_double, _doubleArray, _doubleArray,
    #                            _doubleArray]
    #  _clib.Numerov.restype = c_double

    _clib.Solve1D.argtypes = [c_double, c_int, doubleArray, c_int,
                              doubleArray, doubleArray,
                              POINTER(cBand), doubleArray]
    _clib.Solve1D.restype = c_int

    _clib.FillPsi.argtypes = [c_double, c_int, doubleArray, c_int,
                              doubleArray, doubleArray, doubleArray,
                              intArray, intArray, POINTER(cBand)]
    _clib.FillPsi.restype = None

    _clib.LOphononScatter.argtypes = [c_double, c_int, c_double, doubleArray]
    _clib.LOphononScatter.restype = c_double

    _clib.LOtotal.argtypes = [c_double, c_int, doubleArray, doubleArray,
                              doubleArray, c_int]
    _clib.LOtotal.restype = c_double
    return _clib, Band


_clib, Band = bindOpenMP(False)

#  def cNumerov(step, y0, y1, E, V, m, xmin=0, xmax=None):
#      if not xmax:
#          xmax = V.size
#      if not isinstance(m, np.ndarray):
#          m = m*np.ones(V.size)
#      y = np.empty(xmax-xmin)
#      yend = _clib.Numerov(c_double(step), xmax-xmin,
#                          c_double(y0), c_double(y1), E,
#                          V[xmin:xmax], m[xmin:xmax], y)
#      return yend


def cSimpleSolve1D(step: float, Es: np.ndarray, V: np.ndarray, m: floatOrArray,
                   xmin: int = 0, xmax: typing.Optional[int] = None
                   ) -> np.ndarray:
    """
    Find eigen energies. Assume mass as given.
    """
    if not xmax:
        xmax = V.size
    if not isinstance(m, np.ndarray):
        m = m*np.ones(V.size)
    EigenE = np.empty(Es.size)
    EigenEN = _clib.Solve1D(c_double(step), xmax-xmin, Es, Es.size,
                            V[xmin:xmax], m[xmin:xmax], None, EigenE)
    return EigenE[:EigenEN]


def cSimpleFillPsi(step: float, EigenEs: np.ndarray, V: np.ndarray,
                   m: floatOrArray, xmin: int = 0,
                   xmax: typing.Optional[int] = None) -> np.ndarray:
    """
    Find wave functions. Assume mass as given.
    """
    if not xmax:
        xmax = V.size
    if not isinstance(m, np.ndarray):
        m = m*np.ones(V.size)
    psis = np.empty(EigenEs.size*(xmax-xmin))
    _clib.FillPsi(c_double(step), xmax-xmin, EigenEs, EigenEs.size,
                  V[xmin:xmax], m[xmin:xmax], psis,
                  np.zeros(xmax-xmin, dtype=np.int32),
                  (xmax-xmin)*np.ones(xmax-xmin, dtype=np.int32),
                  None)
    return psis.reshape((EigenEs.size, xmax-xmin))


def cBandSolve1D(step: float, Es: np.ndarray, V: np.ndarray, band: Band,
                 xmin: int = 0, xmax: typing.Optional[int] = None
                 ) -> np.ndarray:
    """
    Find eigen energies using band mass.
    """
    if not xmax:
        xmax = V.size
    EigenE = np.empty(Es.size)
    EigenEN = _clib.Solve1D(c_double(step), xmax-xmin, Es, Es.size,
                            V[xmin:xmax], np.empty(0), band.c, EigenE)
    return EigenE[:EigenEN]


def cBandFillPsi(step: float, EigenEs: np.ndarray, V: np.ndarray, band: Band,
                 xmin: int = 0, xmax: typing.Optional[int] = None,
                 Elower: typing.Optional[float] = None,
                 Eupper: typing.Optional[float] = None,
                 field: typing.Optional[float] = None) -> np.ndarray:
    """
    Find wave functions using band mass. `field`, `Elower` and `Eupper` is used
    only for bound the energy range of the wave functions: outside the bound
    the wavefunction is promised to be zero.
    """
    if not xmax:
        xmax = V.size
    psis = np.empty(EigenEs.size*(xmax-xmin))
    if field is not None:
        starts = np.floor((Elower - EigenEs)/(field*step*1E-5)
                          ).astype(np.int32)
        starts[starts < xmin] = xmin
        ends = np.ceil((Eupper - EigenEs)/(field*step*1E-5)
                       ).astype(np.int32)
        ends[ends > xmax] = xmax
    else:
        starts = np.zeros(xmax-xmin, dtype=np.int32)
        ends = (xmax-xmin)*np.ones(xmax-xmin, dtype=np.int32)
    _clib.FillPsi(c_double(step), xmax-xmin, EigenEs, EigenEs.size,
                  V[xmin:xmax], np.empty(0), psis,
                  starts, ends, band.c)
    return psis.reshape((EigenEs.size, xmax-xmin))


def cLOphononScatter(step: float, kl: float,
                     psi_ij: np.ndarray, xmin: int = 0,
                     xmax: typing.Optional[int] = None) -> float:
    if not xmax:
        xmax = psi_ij.size
    return _clib.LOphononScatter(c_double(step), xmax-xmin,
                                 c_double(kl), psi_ij)


def cLOtotal(step: float, kls: np.ndarray,
             psi_ijs: np.ndarray, fjs: np.ndarray):
    return _clib.LOtotal(c_double(step), psi_ijs.shape[1], kls,
                         psi_ijs.flatten(), fjs, psi_ijs.shape[0])


def isMP() -> bool:
    return _clib.isMP() == 1


if __name__ == "__main__":
    print(_clib.invAlpha())
    if isMP():
        print("Woo! OpenMP is loaded!")

# vim: ts=4 sw=4 sts=4 expandtab



from .QCLayers import QCLayers, SchrodingerLayer
from .OptStrata import OptStrata, MaxwellLayer
try:
    from . import OneDQuantum
except OSError:
    OneDQuantum = None

__all__ = (QCLayers, SchrodingerLayer, OptStrata, MaxwellLayer,
           OneDQuantum)



#!/usr/bin/env python3
# -*- coding:utf-8 -*-

from setuptools import setup, find_packages
from setuptools.command.install import install
from setuptools.command.develop import develop
from wheel.bdist_wheel import bdist_wheel, get_platform
import os
import subprocess
import warnings


_built = False


def build_clib():
    global _built
    if _built:
        return
    _built = True
    cwd = os.getcwd()
    path = os.path.join(os.path.dirname(os.path.abspath(__file__)),
                        'ErwinJr2', 'OneDQuantum')
    os.chdir(path)
    # compile binary
    MSBuild = os.environ.get('MSBUILD')
    if MSBuild is not None:
        sln = 'OneDQuantum.sln'
        makeCMD = [MSBuild, sln, '/p:Configuration=Release']
        makeCMD_MP = [MSBuild, sln, '/p:Configuration=MP_Release']
    else:
        makeCMD = ['make']
        makeCMD_MP = ['make', 'MP']
    try:
        print('call ', makeCMD)
        subprocess.check_call(makeCMD)
    except subprocess.CalledProcessError:
        warnings.warn("Warning: Exit without compiling the C library. "
                      "Features are limited.")
        os.chdir(cwd)
        return
    try:
        print('call ', makeCMD_MP)
        subprocess.check_call(makeCMD_MP)
    except subprocess.CalledProcessError:
        warnings.warn("Warning: openMP not supported")
    os.chdir(cwd)


class EJBdistCMD(bdist_wheel):
    def run(self):
        # This is a hack s.t. the only way to let pip call build and use the
        # result
        print("Building binary for ErwinJr2. (bdist_wheel)")
        build_clib()
        super().run()

    def finalize_options(self):
        super().finalize_options()
        self.plat_name_supplied = True
        self.plat_name = get_platform(self.bdist_dir)


class EJInstallCMD(install):
    def run(self):
        print("Building binary for ErwinJr2. (install)")
        build_clib()
        super().run()


class EJDevelopCMD(develop):
    def run(self):
        print("Building binary for ErwinJr2. (develop)")
        build_clib()
        super().run()


long_description = """
This is a Quantum Cascade Laser (QCL) modeling and design software produced
at Princeton University, Gmachl group.
"""

setup(
    name='ErwinJr2',
    version='2.0.11',
    author='Ming Lyu',
    author_email='minglyu@princeton.edu',
    license="GPL-3.0",
    url='https://princetonuniversity.github.io/ErwinJr2',
    description='A Quantum Cascade Laser Design Tool',
    long_description=long_description,
    long_description_content_type="text/markdown",
    packages=find_packages(),
    package_data={
        'ErwinJr2': [
            'images/*.png', 'images/*.ico', 'images/*.icns',
            'example/PQLiu.json',
            'Info.plist'
        ],
        'ErwinJr2.OneDQuantum': [
            'Makefile', '*.sln', '*.vcxproj',
            '*.c', '*.h', 'fftautocorr/*.c', 'fftautocorr/*.h',
            '*.so', '*.dll', '*.dylib'
        ]
    },
    cmdclass={
        'bdist_wheel': EJBdistCMD,
        'install': EJInstallCMD,
        'develop': EJDevelopCMD
    },
    python_requires='>=3.6',
    install_requires=['numpy>=1.12', 'scipy>=0.18'],
    extras_require={
        'GUI': [
            'pyqt5',
            'matplotlib>=3.3',
            "winshell;platform_system=='Windows'"
        ],
        'dev': [
            # Required by documentation
            'sphinx>=2',
            'sphinx_rtd_theme',
            'sphinxcontrib-bibtex>=2.0.0',
            'breathe'
        ]
    },
    entry_points={
        'console_scripts': [
            'ErwinJr = ErwinJr2.__main__:__main__ [GUI]',
            'ErwinJr-genshortcut = ErwinJr2.genshortcut:create_shortcut [GUI]'
        ]
    }
)



from . import OneDSchrodinger
from .OneDSchrodinger import *
__all__ = (OneDSchrodinger.__all__
           + ['OneDSchrodinger'])

try:
    OneDSchrodinger.bindOpenMP(True)
except OSError:
    OneDSchrodinger.bindOpenMP(False)
